"use strict";

const STARTERS = 'abcdefghijklmnopqrstuvwxyz_';
const CHARS    = 'abcdefghijklmnopqrstuvwxyz0123456789_-';

module.exports = function(settings) {
	var selectors = {
		classes: new Map(),
		ids:     new Map()
	};

	const SELECTOR_PREPEND = settings.prepend || '';

	selectors.classes.prefix =
	selectors.ids.prefix =
		'';

	selectors.classes.prefixLastCharIndex =
	selectors.ids.prefixLastCharIndex =
		-1;

	var external_classes = new Map();
	var external_ids = new Map();

	if (settings.externalSelectors) {
		external_classes = new Map(settings.externalSelectors.c);
		external_ids = new Map(settings.externalSelectors.i);
	}

	this.minifySelector = function(sel, is_a_class) {

		if (is_a_class && external_classes.has(sel)) { return external_classes.get(sel) }
		else if (!is_a_class && external_ids.has(sel)) { return external_ids.get(sel) }

		let key = is_a_class ? 'classes' : 'ids',
			prefix = is_a_class ? '.' : '#';

		var new_sel;

		if (!selectors[key].has(sel)) {
			if (selectors[key].size < STARTERS.length) {
				new_sel = STARTERS[selectors[key].size];
			}

			else {
				let remaining_chars_count = selectors[key].size - STARTERS.length;
				if (remaining_chars_count % CHARS.length == 0) {
					let prefix_is_starter = selectors[key].prefix.length == 1;
					if (prefix_is_starter && selectors[key].prefixLastCharIndex == STARTERS.length - 1 ||
						selectors[key].prefixLastCharIndex == CHARS.length - 1
					) {
						selectors[key].prefix += CHARS[0];
						selectors[key].prefixLastCharIndex = 0;
					} else {
						++selectors[key].prefixLastCharIndex;
						if (prefix_is_starter) {
							selectors[key].prefix = STARTERS[selectors[key].prefixLastCharIndex];
						} else {
							selectors[key].prefix = selectors[key].prefix.slice(0, -1) + CHARS[selectors[key].prefixLastCharIndex];
						}
					}
				}
				new_sel = selectors[key].prefix + CHARS[remaining_chars_count % CHARS.length];
			}

			new_sel = SELECTOR_PREPEND + new_sel;
			selectors[key].set(sel, new_sel);
		}

		return selectors[key].get(sel);

	};

	this.export = function() {
		return {
			c: [...selectors.classes],
			i: [...selectors.ids]
		};
	};
};
