"use strict";

module.exports = function() {
    let table = new Map();
    let unusedKeys = new Set();

    this.set = function(key, value) {
        if (table.has(key)) {
            throw new ReferenceError(`The variable ${key} has already been defined`);
        }
        table.set(key, value);
        unusedKeys.add(key);
    };
    this.get = function(key) {
        if (!table.has(key)) {
            throw new ReferenceError(`The variable ${key} does not exist`);
        }
        unusedKeys.delete(key);
        return table.get(key);
    };
    this.end = function() {
        if (unusedKeys.size) {
            throw new SyntaxError(`The variable "${Array.from(unusedKeys)[0]}" is not used`);
        }
    };
};